@@ -98,7 +98,7 @@ static const char *getWcharCp(void) {
 }
 
 
-doscp_t *cp_open(int codepage)
+doscp_t *cp_open(unsigned int codepage)
 {
 	char dosCp[17];
 	doscp_t *ret;
@@ -107,7 +107,7 @@ doscp_t *cp_open(int codepage)
 
 	if(codepage == 0)
 		codepage = mtools_default_codepage;
-	if(codepage < 0 || codepage > 9999) {
+	if(codepage > 9999) {
 		fprintf(stderr, "Bad codepage %d\n", codepage);
 		return NULL;
 	}
@@ -152,19 +152,19 @@ void cp_close(doscp_t *cp)
 	free(cp);
 }
 
-int dos_to_wchar(doscp_t *cp, const char *dos, wchar_t *wchar, size_t len)
+size_t dos_to_wchar(doscp_t *cp, const char *dos, wchar_t *wchar, size_t len)
 {
-	int r;
+	size_t r;
 	size_t in_len=len;
 	size_t out_len=len*sizeof(wchar_t);
 	wchar_t *dptr=wchar;
 	char *dos2 = (char *) dos; /* Magic to be able to call iconv with its 
 				      buggy prototype */
 	r=iconv(cp->from, &dos2, &in_len, (char **)&dptr, &out_len);
-	if(r < 0)
+	if(r == (size_t) -1)
 		return r;
 	*dptr = L'\0';
-	return dptr-wchar;
+	return (size_t) (dptr-wchar);
 }
 
 /**
@@ -172,10 +172,10 @@ int dos_to_wchar(doscp_t *cp, const char *dos, wchar_t *wchar, size_t len)
  * ensure that dest is large enough.
  * mangled will be set if there has been an untranslatable character.
  */
-static int safe_iconv(iconv_t conv, const wchar_t *wchar, char *dest,
+static size_t safe_iconv(iconv_t conv, const wchar_t *wchar, char *dest,
 		      size_t in_len, size_t out_len, int *mangled)
 {
-	int r;
+	size_t r;
 	unsigned int i;
 	char *dptr = dest;
 	size_t len;
@@ -184,7 +184,7 @@ static int safe_iconv(iconv_t conv, const wchar_t *wchar, char *dest,
 
 	while(in_len > 0 && out_len > 0) {
 		r=iconv(conv, (char**)&wchar, &in_len, &dptr, &out_len);
-		if(r >= 0 || errno != EILSEQ) {
+		if(r == (size_t) -1 || errno != EILSEQ) {
 			/* everything transformed, or error that is _not_ a bad
 			 * character */
 			break;
@@ -201,8 +201,8 @@ static int safe_iconv(iconv_t conv, const wchar_t *wchar, char *dest,
 		out_len--;
 	}
 
-	len = dptr-dest; /* how many dest characters have there been
-			    generated */
+	len = (size_t) (dptr-dest); /* how many dest characters have there been
+				       generated */
 
 	/* eliminate question marks which might have been formed by
 	   untransliterable characters */
@@ -335,7 +335,7 @@ static iconv_t to_native = NULL;
 static void initialize_to_native(void)
 {
 	char *li, *cp;
-	int len;
+	size_t len;
 	if(to_native != NULL)
 		return;
 	li = nl_langinfo(CODESET);
@@ -364,12 +364,12 @@ static void initialize_to_native(void)
  * Convert wchar string to native, converting at most len wchar characters
  * Returns number of generated native characters
  */
-int wchar_to_native(const wchar_t *wchar, char *native, size_t len,
-		    size_t out_len)
+size_t wchar_to_native(const wchar_t *wchar, char *native, size_t len,
+		       size_t out_len)
 {
 #ifdef HAVE_ICONV_H
 	int mangled;
-	int r;
+	size_t r;
 	initialize_to_native();
 	len = wcsnlen(wchar,len);
 	r=safe_iconv(to_native, wchar, native, len, out_len, &mangled);
@@ -381,13 +381,11 @@ int wchar_to_native(const wchar_t *wchar, char *native, size_t len,
 	mbstate_t ps;
 	memset(&ps, 0, sizeof(ps));
 	for(i=0; i<len && wchar[i] != 0; i++) {
-		int r = wcrtomb(dptr, wchar[i], &ps);
-		if(r < 0 && errno == EILSEQ) {
+		size_t r = wcrtomb(dptr, wchar[i], &ps);
+		if(r == (size_t) -1 && errno == EILSEQ) {
 			r=1;
 			*dptr='_';
 		}
-		if(r < 0)
-			return r;
 		dptr+=r;
 	}
 	*dptr='\0';
@@ -400,16 +398,16 @@ int wchar_to_native(const wchar_t *wchar, char *native, size_t len,
  * characters. If end is supplied, stop conversion when source pointer
  * exceeds end. Returns number of generated wchars
  */
-int native_to_wchar(const char *native, wchar_t *wchar, size_t len,
-		    const char *end, int *mangled)
+size_t native_to_wchar(const char *native, wchar_t *wchar, size_t len,
+		       const char *end, int *mangled)
 {
 	mbstate_t ps;
 	unsigned int i;
 	memset(&ps, 0, sizeof(ps));
 
 	for(i=0; i<len && (native < end || !end); i++) {
-		int r = mbrtowc(wchar+i, native, len, &ps);
-		if(r < 0) {
+		size_t r = mbrtowc(wchar+i, native, len, &ps);
+		if(r == (size_t) -1) {
 			/* Unconvertible character. Just pretend it's Latin1
 			   encoded (if valid Latin1 character) or substitute
 			   with an underscore if not
