@@ -45,19 +45,38 @@
 
 #ifdef OS_linux
 #include "linux/hdreg.h"
-
-#define _LINUX_STRING_H_
-#define kdev_t int
 #include "linux/fs.h"
-#undef _LINUX_STRING_H_
 
 #endif
 
+/**
+ * Narrow down quantity of sectors to 32bit quantity, and bail out if
+ * it doesn't fit in 32 bits
+ */
+static uint32_t mt_off_t_to_sectors(mt_off_t raw_sect) {
+	/* Number of sectors must fit into 32bit value */
+	if (raw_sect > UINT32_MAX) {
+		fprintf(stderr, "Too many sectors for FAT %08x%08x\n",
+			(uint32_t)(raw_sect>>32), (uint32_t)raw_sect);
+		exit(1);
+	}
+	return (uint32_t) raw_sect;
+}
 
-static int init_geometry_boot(union bootsector *boot, struct device *dev,
-			      uint8_t sectors0,
-			      uint8_t rate_0, uint8_t rate_any,
-			      unsigned long *tot_sectors, int keepBoot)
+static uint32_t ulong_to_sectors(unsigned long raw_sect) {
+	/* Number of sectors must fit into 32bit value */
+	if (raw_sect > ULONG_MAX) {
+		fprintf(stderr, "Too many sectors for FAT %8lx\n",raw_sect);
+		exit(1);
+	}
+	return (uint32_t) raw_sect;
+}
+
+
+static uint16_t init_geometry_boot(union bootsector *boot, struct device *dev,
+				   uint8_t sectors0,
+				   uint8_t rate_0, uint8_t rate_any,
+				   uint32_t *tot_sectors, int keepBoot)
 {
 	int nb_renum;
 	int sector2;
@@ -77,12 +96,12 @@ static int init_geometry_boot(union bootsector *boot, struct device *dev,
 		set_word(boot->boot.psect, 0);
 		set_dword(boot->boot.bigsect, (uint32_t) *tot_sectors);
 	} else {
-		fprintf(stderr, "Too many sectors %ld\n", *tot_sectors);
+		fprintf(stderr, "Too many sectors %u\n", *tot_sectors);
 		exit(1);
 	}
 
 	if (dev->use_2m & 0x7f){
-		int bootOffset;
+		uint16_t bootOffset;
 		uint8_t j;
 		uint8_t size2;
 		uint16_t i;
@@ -122,7 +141,7 @@ static int init_geometry_boot(union bootsector *boot, struct device *dev,
 			boot->bytes[i++] = size2;
 			sector2 -= (1 << size2) >> 2;
 		}
-		boot->bytes[nb_renum] = ( i - nb_renum - 1 ) / 3;
+		boot->bytes[nb_renum] = (uint8_t)(( i - nb_renum - 1 )/3);
 
 		set_word(boot->boot.ext.old.InfTm, i);
 
@@ -141,7 +160,7 @@ static int init_geometry_boot(union bootsector *boot, struct device *dev,
 		/* checksum */
 		for (sum=0, j=64; j<i; j++)
 			sum += boot->bytes[j];/* checksum */
-		boot->boot.ext.old.CheckSum=-sum;
+		boot->boot.ext.old.CheckSum=(unsigned char)-sum;
 		return bootOffset;
 	} else {
 		if(!keepBoot) {
@@ -325,13 +344,14 @@ static __inline__ void format_root(Fs_t *Fs, char *label, union bootsector *boot
 	if(Fs->fat_bits == 32)
 		set_word(boot->boot.dirents, 0);
 	else
-		set_word(boot->boot.dirents, Fs->dir_len * (Fs->sector_size / 32));
+		set_word(boot->boot.dirents,
+			 (uint16_t) (Fs->dir_len * (Fs->sector_size / 32)));
 	free(buf);
 }
 
 
 #ifdef USE_XDF
-static void xdf_calc_fat_size(Fs_t *Fs, unsigned long tot_sectors,
+static void xdf_calc_fat_size(Fs_t *Fs, uint32_t tot_sectors,
 			      int fat_bits)
 {
 	unsigned int rem_sect;
@@ -359,12 +379,13 @@ static void xdf_calc_fat_size(Fs_t *Fs, unsigned long tot_sectors,
 }
 #endif
 
-static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
+static void calc_fat_size(Fs_t *Fs, uint32_t tot_sectors)
 {
-	unsigned long rem_sect;
-	unsigned long real_rem_sect;
-	unsigned long numerator;
-	unsigned long denominator;
+	uint32_t rem_sect;
+	uint32_t real_rem_sect;
+	uint32_t numerator;
+	uint32_t denominator;
+	uint32_t corr=0; /* correct numeric overflow */
 	unsigned int fat_nybbles;
 	unsigned int slack;
 	int printGrowMsg=1; /* Should we print "growing FAT" messages ?*/
@@ -401,6 +422,12 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 
 	fat_nybbles = Fs->fat_bits / 4;
 	numerator   = rem_sect+2*Fs->cluster_size;
+	/* Might overflow, but will be cancelled out below. As the
+	   operation is unsigned, a posteriori fixup is allowable, as
+	   wrap-around is part of the spec. For *signed* quantities,
+	   this hack would be incorrect, as it would be "undefined
+	   behavior" */
+	
 	denominator =
 	  Fs->cluster_size * Fs->sector_size * 2 +
 	  Fs->num_fat * fat_nybbles;
@@ -412,12 +439,25 @@ static void calc_fat_size(Fs_t *Fs, unsigned long tot_sectors)
 		 * rather than multiplying the numerator */
 		denominator = denominator / fat_nybbles;
 
+	/* Substract denominator from numerator to "cancel out" an
+	   unsigned integer overflow which might have happened with
+	   total number of sectors very near maximum (2^32-1) and huge
+	   cluster size. This substraction removes 1 from the result
+	   of the following division, so we will add 1 again after the
+	   division. However, we only do this if (original) numerator
+	   is bigger than denominator though, as otherwise we risk the
+	   inverse problem of going below 0 on small disks */
+	if(rem_sect > denominator) {
+		numerator -=  denominator;
+		corr++;
+	}
+	
 #ifdef DEBUG
 	fprintf(stderr, "Numerator=%lu denominator=%lu\n",
 		numerator, denominator);
 #endif
 
-	Fs->fat_len = (numerator-1)/denominator+1;
+	Fs->fat_len = (numerator-1)/denominator+1+corr;
 	Fs->num_clus = (rem_sect-(Fs->fat_len*Fs->num_fat))/Fs->cluster_size;
 
 	/* Apply upper bounds for FAT bits */
@@ -503,25 +543,25 @@ static unsigned char bootprog[]=
  0xb9, 0x01, 0x00, 0xcd, 0x13, 0x72, 0x05, 0xea, 0x00, 0x7c, 0x00,
  0x00, 0xcd, 0x19};
 
-static __inline__ void inst_boot_prg(union bootsector *boot, int offset)
+static __inline__ void inst_boot_prg(union bootsector *boot, uint16_t offset)
 {
 	memcpy((char *) boot->boot.jump + offset,
 	       (char *) bootprog, sizeof(bootprog) /sizeof(bootprog[0]));
 	if(offset - 2 < 0x80) {
 	  /* short jump */
 	  boot->boot.jump[0] = 0xeb;
-	  boot->boot.jump[1] = offset -2;
+	  boot->boot.jump[1] = (uint8_t) (offset -2);
 	  boot->boot.jump[2] = 0x90;
 	} else {
 	  /* long jump, if offset is too large */
 	  boot->boot.jump[0] = 0xe9;
-	  boot->boot.jump[1] = offset -3;
-	  boot->boot.jump[2] = 0x00;
+	  boot->boot.jump[1] = (uint8_t) (offset - 3);
+	  boot->boot.jump[2] = (uint8_t) ( (offset - 3) >> 8);
 	}
 	set_word(boot->boot.jump + offset + 20, offset + 24);
 }
 
-static void calc_cluster_size(struct Fs_t *Fs, unsigned long tot_sectors,
+static void calc_cluster_size(struct Fs_t *Fs, uint32_t tot_sectors,
 			      int fat_bits)
 
 {
@@ -568,7 +608,7 @@ static void calc_cluster_size(struct Fs_t *Fs, unsigned long tot_sectors,
 			exit(1);
 	}
 
-	if(tot_sectors <= Fs->fat_start + Fs->num_fat + Fs->dir_len) {
+	if(tot_sectors <= Fs->fat_start + Fs->num_fat + Fs->dir_len + 0u) {
 		/* we need at least enough sectors to fit boot, fat and root
 		 * dir */
 		fprintf(stderr, "Not enough sectors\n");
@@ -607,7 +647,7 @@ static int old_dos_size_to_geom(size_t size,
 }
 
 
-static void calc_fs_parameters(struct device *dev, unsigned long tot_sectors,
+static void calc_fs_parameters(struct device *dev, uint32_t tot_sectors,
 			       struct Fs_t *Fs, union bootsector *boot)
 {
 	struct OldDos_t *params=NULL;
@@ -662,12 +702,12 @@ static void calc_fs_parameters(struct device *dev, unsigned long tot_sectors,
 		}
 	}
 
-	set_word(boot->boot.fatlen, Fs->fat_len);
+	set_word(boot->boot.fatlen, (uint16_t) Fs->fat_len);
 }
 
 
 
-static void calc_fs_parameters_32(unsigned long tot_sectors,
+static void calc_fs_parameters_32(uint32_t tot_sectors,
 				  struct Fs_t *Fs, union bootsector *boot)
 {
 	unsigned long num_clus;
@@ -716,6 +756,13 @@ static void usage(int ret)
 	exit(ret);
 }
 
+static uint32_t ulongToSectors(unsigned long r_sectors) {
+	if(r_sectors > UINT32_MAX) {
+		fprintf(stderr, "Too many sectors %ld\n", r_sectors);
+	}
+	return (uint32_t) r_sectors;
+}
+
 #ifdef OS_linux
 static int get_sector_size(int fd, char *errmsg) {
 	int sec_size;
@@ -734,10 +781,10 @@ static int get_sector_size(int fd, char *errmsg) {
 static int get_block_geom(int fd, struct device *dev, char *errmsg) {
 	struct hd_geometry geom;
 	int sec_size;
-	long size;
+	unsigned long size;
 	uint16_t heads=dev->heads;
 	uint16_t sectors=dev->sectors;
-	unsigned int sect_per_track;
+	uint32_t sect_per_track;
 
 	if (ioctl(fd, HDIO_GETGEO, &geom) < 0) {
 		sprintf(errmsg, "Could not get geometry of device (%s)",
@@ -766,11 +813,11 @@ static int get_block_geom(int fd, struct device *dev, char *errmsg) {
 
 	sect_per_track = heads * sectors;
 	if(!dev->hidden) {
-		unsigned long hidden;
+		uint32_t hidden;
 		hidden = geom.start % sect_per_track;
 		if(hidden && hidden != sectors) {
 			sprintf(errmsg,
-				"Hidden (%ld) does not match sectors (%d)\n",
+				"Hidden (%d) does not match sectors (%d)\n",
 				hidden, sectors);
 			return -1;
 		}
@@ -779,14 +826,14 @@ static int get_block_geom(int fd, struct device *dev, char *errmsg) {
 	dev->heads = heads;
 	dev->sectors = sectors;
 	if(!dev->tracks)
-		dev->tracks = (size + dev->hidden % sect_per_track) / sect_per_track;
+		dev->tracks = ulong_to_sectors((size + dev->hidden % sect_per_track) / sect_per_track);
 	return 0;
 }
 #endif
 
-static int get_lba_geom(Stream_t *Direct, unsigned long tot_sectors, struct device *dev, char *errmsg) {
-	int sect_per_track;
-	unsigned long tracks;
+static int get_lba_geom(Stream_t *Direct, uint32_t tot_sectors, struct device *dev, char *errmsg) {
+	unsigned int sect_per_track;
+	uint32_t tracks;
 
 	/* if one value is already specified we do not want to overwrite it */
 	if (dev->heads || dev->sectors || dev->tracks) {
@@ -798,7 +845,6 @@ static int get_lba_geom(Stream_t *Direct, unsigned long tot_sectors, struct devi
 #ifdef OS_linux
 		int fd;
 		int sec_size;
-		long size;
 		struct MT_STAT stbuf;
 
 		fd = get_fd(Direct);
@@ -808,6 +854,8 @@ static int get_lba_geom(Stream_t *Direct, unsigned long tot_sectors, struct devi
 		}
 
 		if (S_ISBLK(stbuf.st_mode)) {
+			unsigned long size;
+			unsigned long r_sectors;
 			if (ioctl(fd, BLKGETSIZE, &size) != 0) {
 				sprintf(errmsg, "Could not get size of device (%s)",
 					strerror(errno));
@@ -823,11 +871,12 @@ static int get_lba_geom(Stream_t *Direct, unsigned long tot_sectors, struct devi
 				dev->ssize++;
 			}
 			if ((dev->ssize & 0x7f) > 2)
-				tot_sectors = size >> ((dev->ssize & 0x7f) - 2);
+				r_sectors = size >> ((dev->ssize & 0x7f) - 2);
 			else
-				tot_sectors = size << (2 - (dev->ssize & 0x7f));
+				r_sectors = size << (2 - (dev->ssize & 0x7f));
+			tot_sectors = ulongToSectors(r_sectors);
 		} else if (S_ISREG(stbuf.st_mode)) {
-			tot_sectors = stbuf.st_size >> ((dev->ssize & 0x7f) + 7);
+			tot_sectors = mt_off_t_to_sectors(stbuf.st_size >> ((dev->ssize & 0x7f) + 7));
 		} else {
 			sprintf(errmsg, "Could not get size of device (%s)",
 				"No method available");
@@ -860,12 +909,8 @@ static int get_lba_geom(Stream_t *Direct, unsigned long tot_sectors, struct devi
 
 	sect_per_track = dev->heads * dev->sectors;
 	tracks = (tot_sectors + dev->hidden % sect_per_track) / sect_per_track;
-	if (tracks > 0xFFFFFFFF) {
-		sprintf(errmsg, "Device is too big, it has too many tracks");
-		return -1;
-	}
 
-	dev->tracks = (uint32_t) tracks;
+	dev->tracks = tracks;
 
 	return 0;
 }
@@ -883,10 +928,10 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	uint8_t rate_0, rate_any;
 	int mangled;
 	uint8_t argssize=0; /* sector size */
-	int msize=0;
+	uint16_t msize=0;
 	int fat32 = 0;
 	struct label_blk_t *labelBlock;
-	int bootOffset;
+	size_t bootOffset;
 
 #ifdef USE_XDF
 	unsigned int i;
@@ -900,8 +945,8 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	struct device used_dev;
 	unsigned int argtracks;
 	uint16_t argheads, argsectors;
-	unsigned long tot_sectors=0;
-	int blocksize;
+	uint32_t tot_sectors=0;
+	size_t blocksize;
 
 	char drive, name[EXPAND_BUF];
 
@@ -913,17 +958,18 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 
 	uint32_t serial;
  	int serial_set;
-	int fsVersion;
-	int mediaDesc=-1;
-
+	uint16_t fsVersion;
+	uint8_t mediaDesc=0;
+	bool haveMediaDesc=false;
+	
 	mt_size_t maxSize;
 
 	int Atari = 0; /* should we add an Atari-style serial number ? */
 
-	unsigned int backupBoot = 6;
+	uint16_t backupBoot = 6;
 	int backupBootSet = 0;
 
-	unsigned int resvSects = 0;
+	uint8_t resvSects = 0;
 	
 	char *endptr;
 
@@ -942,14 +988,14 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	Fs.refs = 1;
 	Fs.dir_len = 0;
 	if(getenv("MTOOLS_DIR_LEN")) {
-	  Fs.dir_len = atoui(getenv("MTOOLS_DIR_LEN"));
+		Fs.dir_len = atou16(getenv("MTOOLS_DIR_LEN"));
 	  if(Fs.dir_len <= 0)
 	    Fs.dir_len=0;
 	}
 	Fs.fat_len = 0;
 	Fs.num_fat = 2;
 	if(getenv("MTOOLS_NFATS")) {
-	  Fs.num_fat = atoui(getenv("MTOOLS_NFATS"));
+		Fs.num_fat = atou8(getenv("MTOOLS_NFATS"));
 	  if(Fs.num_fat <= 0)
 	    Fs.num_fat=2;
 	}
@@ -963,6 +1009,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	while ((c = getopt(argc,argv,
 			   "i:148f:t:n:v:qub"
 			   "kK:R:B:r:L:I:FCc:Xh:s:T:l:N:H:M:S:2:30:Aad:m:"))!= EOF) {
+		errno = 0;
 		endptr = NULL;
 		switch (c) {
 			case 'i':
@@ -1059,7 +1106,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 				break;
 
 			case 'M':
-				msize = atoi(optarg);
+				msize = atou16(optarg);
 				if(msize != 512 &&
 				   msize != 1024 &&
 				   msize != 2048 &&
@@ -1087,7 +1134,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 				break;
 
 			case 'I':
-				fsVersion = strtoi(optarg,&endptr,0);
+				fsVersion = strtou16(optarg,&endptr,0);
 				break;
 
 			case 'c':
@@ -1095,7 +1142,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 				break;
 
 			case 'r':
-				Fs.dir_len = strtoui(optarg,&endptr,0);
+				Fs.dir_len = strtou16(optarg,&endptr,0);
 				break;
 			case 'L':
 				Fs.fat_len = strtoui(optarg,&endptr,0);
@@ -1109,7 +1156,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 				keepBoot = 1;
 				break;
 			case 'K':
-				backupBoot = atoui(optarg);
+				backupBoot = atou16(optarg);
 				backupBootSet=1;
 				if(backupBoot < 2) {
 				  fprintf(stderr, "Backupboot must be greater than 2\n");
@@ -1117,26 +1164,28 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 				}
 				break;
 			case 'R':
-				resvSects = atoui(optarg);
+				resvSects = atou8(optarg);
 				break;
 			case 'h':
 				argheads = atou16(optarg);
 				break;
 			case 'd':
-				Fs.num_fat = atoui(optarg);
+				Fs.num_fat = atou8(optarg);
 				break;
 			case 'm':
-				mediaDesc = strtoi(optarg,&endptr,0);
+				mediaDesc = strtou8(optarg,&endptr,0);
 				if(*endptr)
-					mediaDesc = strtoi(optarg,&endptr,16);
+					mediaDesc = strtou8(optarg,&endptr,16);
+				if(optarg == endptr || *endptr) {
+				  fprintf(stderr, "Bad mediadesc %s\n", optarg);
+				  exit(1);
+				}
+				haveMediaDesc=true;
 				break;
 			default:
 				usage(1);
 		}
-		if(endptr && *endptr) {
-			fprintf(stderr, "Bad number %s\n", optarg);
-			exit(1);
-		}
+		check_number_parse_errno((char)c, optarg, endptr);
 	}
 
 	if (argc - optind > 1)
@@ -1267,7 +1316,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 #endif
 		Fs.sector_size = 512;
 		if( !(used_dev.use_2m & 0x7f)) {
-			Fs.sector_size = 128 << (used_dev.ssize & 0x7f);
+			Fs.sector_size = (uint16_t) (128u << (used_dev.ssize & 0x7f));
 		}
 
 		SET_INT(Fs.sector_size, msize);
@@ -1318,13 +1367,11 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 
 	/* calculate the total number of sectors */
 	if(tot_sectors == 0) {
-		unsigned long sect_per_track = used_dev.heads*used_dev.sectors;
-		tot_sectors = used_dev.tracks*sect_per_track - used_dev.hidden%sect_per_track;
-		/* Number of sectors must fit into 32bit value */
-		if (tot_sectors > 0xFFFFFFFF) {
-			fprintf(stderr, "Too many sectors\n");
-			exit(1);
-		}
+		uint32_t sect_per_track = used_dev.heads*used_dev.sectors;
+		mt_off_t rtot_sectors =
+			used_dev.tracks*(mt_off_t)sect_per_track -
+			used_dev.hidden%sect_per_track;
+		tot_sectors = mt_off_t_to_sectors(rtot_sectors);
 	}
 
 	/* create the image file if needed */
@@ -1337,13 +1384,15 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	/* the boot sector */
 	if(bootSector) {
 		int fd;
-
+		ssize_t ret;
+		
 		fd = open(bootSector, O_RDONLY | O_BINARY | O_LARGEFILE);
 		if(fd < 0) {
 			perror("open boot sector");
 			exit(1);
 		}
-		if(read(fd, &boot.bytes, blocksize) < blocksize) {
+		ret=read(fd, &boot.bytes, blocksize);
+		if(ret < 0 || (size_t) ret < blocksize) {
 			perror("short read on boot sector");
 			exit(1);
 		}
@@ -1378,7 +1427,8 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 			setBeginEnd(&partTable[1], 0,
 				    used_dev.heads * used_dev.sectors *
 				    used_dev.tracks,
-				    used_dev.heads, used_dev.sectors, 1, 0,
+				    (uint8_t) used_dev.heads,
+				    (uint8_t) used_dev.sectors, 1, 0,
 				    used_dev.fat_bits);
 		}
 	}
@@ -1478,7 +1528,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 					rate_0, rate_any,
 					&tot_sectors, keepBoot);
 	if(!bootOffset) {
-		bootOffset = ((unsigned char *) labelBlock) - boot.bytes +
+		bootOffset = ptrdiff((char *) labelBlock, (char*)boot.bytes) +
 			sizeof(struct label_blk_t);
 	}
 	if(Atari) {
@@ -1488,8 +1538,8 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 		boot.boot.banner[7] = (char) random();
 	}
 
-	if(!keepBoot)
-		inst_boot_prg(&boot, bootOffset);
+	if(!keepBoot && bootOffset <= UINT16_MAX)
+		inst_boot_prg(&boot, (uint16_t)bootOffset);
 	/* Mimic 3.8 behavior, else 2m disk do not work (???)
 	 * luferbu@fluidsignal.com (Luis Bustamante), Fri, 14 Jun 2002
 	 */
@@ -1500,7 +1550,7 @@ void mformat(int argc, char **argv, int dummy UNUSEDP)
 	}
 	if(used_dev.use_2m & 0x7f)
 		Fs.num_fat = 1;
-	if(mediaDesc != -1)
+	if(haveMediaDesc)
 		boot.boot.descr=mediaDesc;
 	Fs.lastFatSectorNr = 0;
 	Fs.lastFatSectorData = 0;
